package kr.co.vividnext.sodalive.admin.calculate

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.use.QUseCan.useCan
import kr.co.vividnext.sodalive.can.use.QUseCanCalculate.useCanCalculate
import kr.co.vividnext.sodalive.can.use.UseCanCalculateStatus
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class AdminCalculateQueryRepository(private val queryFactory: JPAQueryFactory) {
    fun getCalculateLive(startDate: LocalDateTime, endDate: LocalDateTime): List<GetCalculateLiveQueryData> {
        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                liveRoom.beginDateTime,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )

        return queryFactory
            .select(
                QGetCalculateLiveQueryData(
                    member.email,
                    member.nickname,
                    formattedDate,
                    liveRoom.title,
                    liveRoom.price,
                    useCan.canUsage,
                    useCanCalculate.id.count(),
                    useCanCalculate.can.sum()
                )
            )
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.room, liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(
                useCanCalculate.status.eq(UseCanCalculateStatus.RECEIVED)
                    .and(liveRoom.beginDateTime.goe(startDate))
                    .and(liveRoom.beginDateTime.loe(endDate))
            )
            .groupBy(liveRoom.id, useCan.canUsage)
            .orderBy(member.nickname.desc(), formattedDate.desc())
            .fetch()
    }
}
