package kr.co.vividnext.sodalive.admin.calculate

import kr.co.vividnext.sodalive.can.use.CanUsage
import org.springframework.stereotype.Service
import java.time.LocalDate
import java.time.ZoneId
import java.time.format.DateTimeFormatter
import kotlin.math.roundToInt

@Service
class AdminCalculateService(private val repository: AdminCalculateQueryRepository) {
    fun getCalculateLive(startDateStr: String, endDateStr: String): List<GetCalculateLiveResponse> {
        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
        val startDate = LocalDate.parse(startDateStr, dateTimeFormatter).atTime(0, 0, 0)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val endDate = LocalDate.parse(endDateStr, dateTimeFormatter).atTime(23, 59, 59)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        return repository
            .getCalculateLive(startDate, endDate)
            .asSequence()
            .map {
                val canUsageStr = if (it.canUsage == CanUsage.LIVE) {
                    "유료"
                } else {
                    "후원"
                }

                val numberOfPeople = if (it.canUsage == CanUsage.LIVE) {
                    it.memberCount.toInt()
                } else {
                    0
                }

                // 원화 = totalCoin * 100 ( 캔 1개 = 100원 )
                val totalKrw = it.totalAmount * 100

                // 결제수수료 : 6.6%
                val paymentFee = totalKrw * 0.066f

                // 정산금액 = (원화 - 결제수수료) 의 70%
                val settlementAmount = (totalKrw.toFloat() - paymentFee) * 0.7

                // 원천세 = 정산금액의 3.3%
                val tax = settlementAmount * 0.033

                // 입금액
                val depositAmount = settlementAmount - tax

                GetCalculateLiveResponse(
                    email = it.email,
                    nickname = it.nickname,
                    date = it.date,
                    title = it.title,
                    entranceFee = it.entranceFee,
                    canUsageStr = canUsageStr,
                    numberOfPeople = numberOfPeople,
                    totalAmount = it.totalAmount,
                    totalKrw = totalKrw,
                    paymentFee = paymentFee.roundToInt(),
                    settlementAmount = settlementAmount.roundToInt(),
                    tax = tax.roundToInt(),
                    depositAmount = depositAmount.roundToInt()
                )
            }
            .toList()
    }
}
