package kr.co.vividnext.sodalive.can.charge.free

import kr.co.vividnext.sodalive.can.charge.Charge
import kr.co.vividnext.sodalive.can.charge.ChargeRepository
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.payment.Payment
import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.common.AdsChargeException
import kr.co.vividnext.sodalive.fcm.FcmEvent
import kr.co.vividnext.sodalive.fcm.FcmEventType
import kr.co.vividnext.sodalive.member.MemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdsChargeService(
    private val repository: AdsChargeRepository,
    private val memberRepository: MemberRepository,
    private val chargeRepository: ChargeRepository,
    private val applicationEventPublisher: ApplicationEventPublisher,

    @Value("\${point-click.placement-uid}")
    private val placementUid: String
) {
    @Transactional
    fun adsCharge(
        transactionKey: String,
        placementUid: String,
        adKey: String,
        adName: String,
        adProfit: String,
        adCurrency: String,
        point: String,
        deviceIfa: String,
        memberId: String
    ) {
        if (placementUid != this.placementUid) {
            throw AdsChargeException("잘못된 요청입니다.")
        }

        val member = memberRepository.findByIdOrNull(id = memberId.toLong())
            ?: throw AdsChargeException("잘못된 요청입니다.")

        val adsCharge = AdsCharge(
            transactionKey = transactionKey,
            adKey = adKey,
            adName = adName,
            adProfit = adProfit.toFloat(),
            adCurrency = adCurrency,
            point = point.toFloat(),
            deviceIfa = deviceIfa
        )
        adsCharge.member = member
        repository.save(adsCharge)

        val charge = Charge(0, rewardCan = adsCharge.point.toInt(), status = ChargeStatus.ADS)
        charge.title = "${adsCharge.point.toInt()} 캔"
        charge.member = member

        val payment = Payment(
            status = PaymentStatus.COMPLETE,
            paymentGateway = PaymentGateway.POINT_CLICK_AD
        )

        payment.method = "제휴보상"
        charge.payment = payment
        chargeRepository.save(charge)

        member.charge(0, adsCharge.point.toInt(), "ads")

        applicationEventPublisher.publishEvent(
            FcmEvent(
                type = FcmEventType.INDIVIDUAL,
                title = "제휴보상",
                message = "${adsCharge.point.toInt()} 캔이 지급되었습니다.",
                recipients = listOf(member.id!!),
                isAuth = false
            )
        )
    }
}
