package kr.co.vividnext.sodalive.creator.admin.content

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@PreAuthorize("hasRole('CREATOR')")
@RequestMapping("/creator-admin/audio-content")
class CreatorAdminContentController(private val service: CreatorAdminContentService) {
    @GetMapping("/list")
    fun getAudioContentList(
        pageable: Pageable,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getAudioContentList(pageable, member))
    }

    @GetMapping("/search")
    fun searchAudioContent(
        @RequestParam(value = "search_word") searchWord: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.searchAudioContent(searchWord, member, pageable))
    }

    @PutMapping
    fun modifyAudioContent(
        @RequestPart("coverImage", required = false) coverImage: MultipartFile? = null,
        @RequestPart("request") requestString: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.updateAudioContent(coverImage, requestString, member))
    }
}
