package kr.co.vividnext.sodalive.creator.admin.member

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.login.LoginRequest
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestHeader
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/creator-admin/member")
class CreatorAdminMemberController(private val service: CreatorAdminMemberService) {
    @PostMapping("/login")
    fun login(@RequestBody loginRequest: LoginRequest) = service.login(loginRequest)

    @PostMapping("/logout")
    @PreAuthorize("hasRole('CREATOR')")
    fun logout(
        @RequestHeader("Authorization") token: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.logout(token.removePrefix("Bearer "), member.id!!))
    }
}
