package kr.co.vividnext.sodalive.explorer

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.explorer.profile.PostCreatorNoticeRequest
import kr.co.vividnext.sodalive.explorer.profile.PostWriteCheersRequest
import kr.co.vividnext.sodalive.explorer.profile.PutWriteCheersRequest
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/explorer")
class ExplorerController(private val service: ExplorerService) {
    @GetMapping
    fun getExplorer(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getExplorer(member))
    }

    @GetMapping("/search/channel")
    fun getSearchChannel(
        @RequestParam channel: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.getSearchChannel(channel, member))
    }

    @GetMapping("/profile/{id}")
    fun getCreatorProfile(
        @PathVariable("id") creatorId: Long,
        @RequestParam timezone: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.getCreatorProfile(creatorId, timezone, member))
    }

    @GetMapping("/profile/{id}/donation-rank")
    fun getCreatorProfileDonationRanking(
        @PathVariable("id") creatorId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.getCreatorProfileDonationRanking(creatorId, pageable, member))
    }

    @PostMapping("/profile/cheers")
    fun writeCheers(
        @RequestBody request: PostWriteCheersRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.writeCheers(request, member), "등록되었습니다.")
    }

    @PutMapping("/profile/cheers")
    fun modifyCheers(
        @RequestBody request: PutWriteCheersRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.modifyCheers(request, member), "수정되었습니다.")
    }

    @PostMapping("/profile/notice")
    @PreAuthorize("hasAnyRole('CREATOR')")
    fun postCreatorNotice(
        @RequestBody request: PostCreatorNoticeRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.saveNotice(member, request.notice), "공지사항이 저장되었습니다.")
    }

    @GetMapping("/profile/{id}/follower-list")
    fun getFollowerList(
        @PathVariable("id") creatorId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.getFollowerList(creatorId, member, pageable))
    }

    @GetMapping("/profile/{id}/cheers")
    fun getCreatorProfileCheers(
        @PathVariable("id") creatorId: Long,
        @RequestParam timezone: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.getCreatorProfileCheers(creatorId, timezone, pageable))
    }
}
