package kr.co.vividnext.sodalive.live.recommend

import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service

@Service
class LiveRecommendService(
    private val repository: LiveRecommendRepository,
    private val blockMemberRepository: BlockMemberRepository
) {

    fun getRecommendLive(member: Member): List<GetRecommendLiveResponse> {
        return repository.getRecommendLive(
            memberId = member.id!!,
            isBlocked = { blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it) },
            isAdult = member.auth != null
        )
    }

    fun getRecommendChannelList(member: Member): List<GetRecommendChannelResponse> {
        val onAirChannelList = repository.getOnAirRecommendChannelList(
            member.id!!,
            isBlocked = { blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it) },
            isAdult = member.auth != null
        )

        if (onAirChannelList.size >= 20) {
            return onAirChannelList
        }

        val onAirCreatorIdList = onAirChannelList.asSequence()
            .map { it.creatorId }
            .toList()

        val notOnAirCreatorList = repository.getRecommendChannelList(
            member.id!!,
            withOutCreatorList = onAirCreatorIdList,
            limit = (20 - onAirChannelList.size).toLong(),
            isBlocked = { blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it) }
        )

        return onAirChannelList + notOnAirCreatorList
    }

    fun getFollowingChannelList(member: Member): List<GetRecommendChannelResponse> {
        val onAirFollowingChannelList = repository.getOnAirFollowingChannelList(
            memberId = member.id!!,
            isBlocked = { blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it) },
            isAdult = member.auth != null
        )

        if (onAirFollowingChannelList.size >= 20) {
            return onAirFollowingChannelList
        }

        val onAirCreatorIdList = onAirFollowingChannelList.asSequence()
            .map { it.creatorId }
            .toList()

        val notOnAirFollowingChannelList = repository.getFollowingChannelList(
            memberId = member.id!!,
            withOutCreatorList = onAirCreatorIdList,
            limit = (20 - onAirCreatorIdList.size).toLong(),
            isBlocked = { blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it) }
        )

        return onAirFollowingChannelList + notOnAirFollowingChannelList
    }

    fun getFollowingAllChannelList(member: Member, pageable: Pageable): GetCreatorFollowingAllListResponse {
        val totalCount = repository.getCreatorFollowingAllListTotalCount(
            memberId = member.id!!,
            isBlocked = { blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it) }
        )

        val items = repository.getCreatorFollowingAllList(
            memberId = member.id!!,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong(),
            isBlocked = { blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it) }
        )

        return GetCreatorFollowingAllListResponse(
            totalCount = totalCount,
            items = items
        )
    }
}
