package kr.co.vividnext.sodalive.live.reservation

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/live/reservation")
class LiveReservationController(private val service: LiveReservationService) {
    @PostMapping
    fun makeReservation(
        @RequestBody request: MakeLiveReservationRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.makeReservation(request, member.id!!))
    }

    @GetMapping
    fun getReservationList(
        @RequestParam isActive: Boolean,
        @RequestParam(value = "timezone") timezone: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.getReservationList(member.id!!, isActive, timezone))
    }

    @GetMapping("/{id}")
    fun getReservation(
        @PathVariable id: Long,
        @RequestParam(value = "timezone") timezone: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.getReservation(id, member.id!!, timezone))
    }

    @PutMapping("/cancel")
    fun cancelReservation(
        @RequestBody request: CancelLiveReservationRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.cancelReservation(request, member.id!!))
    }
}
